import test from "tape";
import Perj from "../src/perj.js";
import { Tool } from "./tool.js";
import { data } from "../data/index.js";

const tool = new Tool();
const write = tool.write.bind(tool);
const passThrough = true;
const serializerTardis = { tardis: tardisSerializer };
const serializerSerenity = { serenity: serenitySerializer };
const serializerBoth = { tardis: tardisSerializer, serenity: serenitySerializer };

test("object serialize tests", (t) => {
  t.test(`${t.name}: first serializer test`, (t) => {
    tool.reset();
    let log = new Perj({ serializers: serializerTardis, passThrough, write });
    log.info("tardis", { tardis: data.tardis });
    t.equal(Object.keys(tool.jsonOut.data).length, 1);
    t.equal(Object.keys(tool.jsonOut.data.tardis).length, 3);
    t.equal(tool.getType(tool.jsonOut.data.tardis), "Object");
    t.equal(tool.getType(tool.objOut.data.tardis), "Object");
    t.equal(tool.jsonOut.data.tardis.name, data.tardis.name);
    t.equal(tool.objOut.data.tardis.name, data.tardis.name);
    t.deepEqual(tool.jsonOut.data.tardis.features, data.tardis.features);
    t.deepEqual(tool.objOut.data.tardis.features, data.tardis.features);
    t.deepEqual(tool.jsonOut.data.tardis.exterior, data.tardis.exterior);
    t.deepEqual(tool.objOut.data.tardis.exterior, data.tardis.exterior);
    t.ok(tool.jsonOut.data.tardis.manufacturer === undefined);
    t.ok(tool.objOut.data.tardis.manufacturer === undefined);
    t.end();
  });
  t.test(`${t.name}: second serializer test`, (t) => {
    tool.reset();
    let log = new Perj({ serializers: serializerSerenity, passThrough, write });
    log.info("serenity", { serenity: data.serenity });
    t.equal(Object.keys(tool.jsonOut.data).length, 1);
    t.equal(Object.keys(tool.jsonOut.data.serenity).length, 3);
    t.equal(tool.getType(tool.jsonOut.data.serenity), "Object");
    t.equal(tool.getType(tool.objOut.data.serenity), "Object");
    t.equal(tool.jsonOut.data.serenity.classCode, data.serenity.classCode);
    t.equal(tool.objOut.data.serenity.classCode, data.serenity.classCode);
    t.equal(tool.jsonOut.data.serenity.engine, data.serenity.engine);
    t.equal(tool.objOut.data.serenity.engine, data.serenity.engine);
    t.deepEqual(tool.jsonOut.data.serenity.upper, data.serenity.interior.upperDeck);
    t.deepEqual(tool.objOut.data.serenity.upper, data.serenity.interior.upperDeck);
    t.ok(tool.jsonOut.data.serenity.url === undefined);
    t.ok(tool.objOut.data.serenity.url === undefined);
    t.end();
  });
  t.test(`${t.name}: two serializers test`, (t) => {
    tool.reset();
    let log = new Perj({ serializers: serializerBoth, passThrough, write });
    log.info("tardis", { tardis: data.tardis });
    t.equal(Object.keys(tool.jsonOut.data).length, 1);
    t.equal(Object.keys(tool.jsonOut.data.tardis).length, 3);
    t.equal(tool.getType(tool.jsonOut.data.tardis), "Object");
    t.equal(tool.getType(tool.objOut.data.tardis), "Object");
    t.equal(tool.jsonOut.data.tardis.name, data.tardis.name);
    t.equal(tool.objOut.data.tardis.name, data.tardis.name);
    t.deepEqual(tool.jsonOut.data.tardis.features, data.tardis.features);
    t.deepEqual(tool.objOut.data.tardis.features, data.tardis.features);
    t.deepEqual(tool.jsonOut.data.tardis.exterior, data.tardis.exterior);
    t.deepEqual(tool.objOut.data.tardis.exterior, data.tardis.exterior);
    t.ok(tool.jsonOut.data.tardis.manufacturer === undefined);
    t.ok(tool.objOut.data.tardis.manufacturer === undefined);
    log.info("serenity", { serenity: data.serenity });
    t.equal(Object.keys(tool.jsonOut.data).length, 1);
    t.equal(Object.keys(tool.jsonOut.data.serenity).length, 3);
    t.equal(tool.getType(tool.jsonOut.data.serenity), "Object");
    t.equal(tool.getType(tool.objOut.data.serenity), "Object");
    t.equal(tool.jsonOut.data.serenity.classCode, data.serenity.classCode);
    t.equal(tool.objOut.data.serenity.classCode, data.serenity.classCode);
    t.equal(tool.jsonOut.data.serenity.engine, data.serenity.engine);
    t.equal(tool.objOut.data.serenity.engine, data.serenity.engine);
    t.deepEqual(tool.jsonOut.data.serenity.upper, data.serenity.interior.upperDeck);
    t.deepEqual(tool.objOut.data.serenity.upper, data.serenity.interior.upperDeck);
    t.ok(tool.jsonOut.data.serenity.url === undefined);
    t.ok(tool.objOut.data.serenity.url === undefined);
    t.end();
  });
  t.test(`${t.name}: serializer with other objects test`, (t) => {
    tool.reset();
    let log = new Perj({ serializers: serializerTardis, passThrough, write });
    log.info("death star", data.deathStar);
    t.equal(Object.keys(tool.jsonOut.data).length, 5);
    t.equal(tool.getType(tool.jsonOut.data), "Object");
    t.equal(tool.getType(tool.objOut.data), "Object");
    t.equal(tool.jsonOut.data.url, data.deathStar.url);
    t.equal(tool.objOut.data.url, data.deathStar.url);
    t.deepEqual(tool.jsonOut.data.production, data.deathStar.production);
    t.deepEqual(tool.objOut.data.production, data.deathStar.production);
    t.deepEqual(tool.jsonOut.data.specifications, data.deathStar.specifications);
    t.deepEqual(tool.objOut.data.specifications, data.deathStar.specifications);
    t.deepEqual(tool.jsonOut.data.locationInformation, data.deathStar.locationInformation);
    t.deepEqual(tool.objOut.data.locationInformation, data.deathStar.locationInformation);
    t.deepEqual(tool.jsonOut.data.usage, data.deathStar.usage);
    t.deepEqual(tool.objOut.data.usage, data.deathStar.usage);
    t.end();
  });
  t.test(`${t.name}: serializer with two objects test`, (t) => {
    tool.reset();
    let log = new Perj({ serializers: serializerTardis, passThrough, write });
    log.info("tardis", { tardis: data.tardis }, data.deathStar);
    t.equal(tool.jsonOut.data.length, 2);
    t.equal(Object.keys(tool.jsonOut.data[0]).length, 1);
    t.equal(Object.keys(tool.jsonOut.data[0].tardis).length, 3);
    t.equal(tool.getType(tool.jsonOut.data[0].tardis), "Object");
    t.equal(tool.getType(tool.objOut.data[0].tardis), "Object");
    t.equal(tool.jsonOut.data[0].tardis.name, data.tardis.name);
    t.equal(tool.objOut.data[0].tardis.name, data.tardis.name);
    t.deepEqual(tool.jsonOut.data[0].tardis.features, data.tardis.features);
    t.deepEqual(tool.objOut.data[0].tardis.features, data.tardis.features);
    t.deepEqual(tool.jsonOut.data[0].tardis.exterior, data.tardis.exterior);
    t.deepEqual(tool.objOut.data[0].tardis.exterior, data.tardis.exterior);
    t.ok(tool.jsonOut.data[0].tardis.manufacturer === undefined);
    t.ok(tool.objOut.data[0].tardis.manufacturer === undefined);
    t.equal(Object.keys(tool.jsonOut.data[1]).length, 5);
    t.equal(tool.getType(tool.jsonOut.data[1]), "Object");
    t.equal(tool.getType(tool.objOut.data[1]), "Object");
    t.equal(tool.jsonOut.data[1].url, data.deathStar.url);
    t.equal(tool.objOut.data[1].url, data.deathStar.url);
    t.deepEqual(tool.jsonOut.data[1].production, data.deathStar.production);
    t.deepEqual(tool.objOut.data[1].production, data.deathStar.production);
    t.deepEqual(tool.jsonOut.data[1].specifications, data.deathStar.specifications);
    t.deepEqual(tool.objOut.data[1].specifications, data.deathStar.specifications);
    t.deepEqual(tool.jsonOut.data[1].locationInformation, data.deathStar.locationInformation);
    t.deepEqual(tool.objOut.data[1].locationInformation, data.deathStar.locationInformation);
    t.deepEqual(tool.jsonOut.data[1].usage, data.deathStar.usage);
    t.deepEqual(tool.objOut.data[1].usage, data.deathStar.usage);
    t.end();
  });
  t.test(`${t.name}: serializer with child test`, (t) => {
    tool.reset();
    let log = new Perj({ serializers: serializerTardis, passThrough, write });
    let child = log.child({ foo: "bar" });
    child.info("tardis", { tardis: data.tardis });
    t.equal(Object.keys(tool.jsonOut.data).length, 1);
    t.equal(Object.keys(tool.jsonOut.data.tardis).length, 3);
    t.equal(tool.getType(tool.jsonOut.data.tardis), "Object");
    t.equal(tool.getType(tool.objOut.data.tardis), "Object");
    t.equal(tool.jsonOut.data.tardis.name, data.tardis.name);
    t.equal(tool.objOut.data.tardis.name, data.tardis.name);
    t.deepEqual(tool.jsonOut.data.tardis.features, data.tardis.features);
    t.deepEqual(tool.objOut.data.tardis.features, data.tardis.features);
    t.deepEqual(tool.jsonOut.data.tardis.exterior, data.tardis.exterior);
    t.deepEqual(tool.objOut.data.tardis.exterior, data.tardis.exterior);
    t.ok(tool.jsonOut.data.tardis.manufacturer === undefined);
    t.ok(tool.objOut.data.tardis.manufacturer === undefined);
    t.end();
  });
  t.test(`${t.name}: serializer type test`, (t) => {
    tool.reset();
    let log = new Perj({ serializers: serializerTardis, passThrough, write });
    log.info(null);
    t.equal(tool.getType(tool.jsonOut.data), "Null", "Data output should be Null");
    t.equal(tool.getType(tool.objOut.data), "Null", "Data output should be Null");
    log.info("message", null);
    t.equal(tool.getType(tool.jsonOut.data), "Null", "Data output should be Null");
    t.equal(tool.getType(tool.objOut.data), "Null", "Data output should be Null");
    log.info(null, "message");
    t.equal(tool.getType(tool.jsonOut.data), "Null", "Data output should be Null");
    t.equal(tool.getType(tool.objOut.data), "Null", "Data output should be Null");
    log.info(undefined);
    t.equal(tool.getType(tool.jsonOut.data), "Null", "Data output should be Null");
    t.equal(tool.getType(tool.objOut.data), "Null", "Data output should be Null");
    log.info(undefined, "message");
    t.equal(tool.getType(tool.jsonOut.data), "Null", "Data output should be Null");
    t.equal(tool.getType(tool.objOut.data), "Null", "Data output should be Null");
    log.info("message", undefined);
    t.equal(tool.getType(tool.jsonOut.data), "Null", "Data output should be Null");
    t.equal(tool.getType(tool.objOut.data), "Null", "Data output should be Null");
    log.info(undefined, null);
    t.equal(tool.getType(tool.jsonOut.data[0]), "Null", "Data output should be Null");
    t.equal(tool.getType(tool.objOut.data[0]), "Null", "Data output should be Null");
    t.equal(tool.getType(tool.jsonOut.data[1]), "Null", "Data output should be Null");
    t.equal(tool.getType(tool.objOut.data[1]), "Null", "Data output should be Null");
    log.info(null, undefined);
    t.equal(tool.getType(tool.jsonOut.data[0]), "Null", "Data output should be Null");
    t.equal(tool.getType(tool.objOut.data[0]), "Null", "Data output should be Null");
    t.equal(tool.getType(tool.jsonOut.data[1]), "Null", "Data output should be Null");
    t.equal(tool.getType(tool.objOut.data[1]), "Null", "Data output should be Null");
    t.end();
  });
  t.end();
});

function tardisSerializer(value) {
  const { name, features, exterior } = value;
  return { name, features, exterior };
}

function serenitySerializer(value) {
  const { classCode, engine } = value;
  const upper = value.interior.upperDeck;
  return { classCode, engine, upper };
}
